/*
Condor_render - submits maya scene file to be rendered under condor
Author: Colin Stolley, 2001
*/
#include "c_render.h"


c_render::c_render() {
	init();
}

c_render::c_render(char* mayafile) {
	init();
	this->mayafile = mayafile;
	startFrame = endFrame = 1; // to make things happy
}

void c_render::init() {
	
	startFrame = endFrame = -1.0; // set these to a bad value so we know when they've been set right
	char* temp = getenv("CONDOR_RENDER_PATH");
	if (temp == NULL) {
		Condor_submit_hdr_file = new char[strlen(CONDOR_RENDER_PATH_DEFAULT CONDOR_SUBMIT_HDR)+1];
		strcpy(Condor_submit_hdr_file, CONDOR_RENDER_PATH_DEFAULT CONDOR_SUBMIT_HDR);
	} else {
		Condor_submit_hdr_file = new char[strlen(temp) + strlen(CONDOR_SUBMIT_HDR)+1];
		strcpy(Condor_submit_hdr_file, temp);
		strcat(Condor_submit_hdr_file, CONDOR_SUBMIT_HDR);
	}

	// set default values
	file_ref_cnt = texture_ref_cnt = transfer_files_cnt = 0;
	output_file_name = "Frame";
	byFrame = 1.0;	
	jobSize = 10;
	mayafile = NULL;
}

c_render::~c_render() {
	int i;
	
	for(i=0; i<texture_ref_cnt; i++) {
		delete[] texture_refs[i];
	}
	
	for(i=0; i<file_ref_cnt; i++) {
		delete[] file_refs[i];
	}
	
	for(i=0; i<transfer_files_cnt; i++) {
		delete[] transfer_files[i];
	}

	delete[] Condor_submit_hdr_file;
	delete MFS;
}

void c_render::usage() {
	printf( "Usage: condorRender -s <float>  -e <float> [-j <int>] [-b <float>]  mayafile\n" );
    printf( "Where:\n-s\t\tStart frame in rendered sequence\n" ); 
    printf( "-e\t\tEnd frame in rendered sequence\n" );
	printf( "-j\t\tNumber of frames per condor job\n" );
	printf( "-b\t\tBy frame for rendered sequence\n" ); 
    printf( "mayafile\tMaya Scene file to be rendered\n" );
};

// Parses command line arguments
// 1 = args parsed and good, 0 = args are bad
int c_render::parseCommandArgs(int argc, char *argv[]) {
	
	for (int i=1; i<argc;) {
		if ( (argv[i][0] == '-') && 
			(strlen(argv[i]) > 1) && 
			(argc > i+1) ) {
			switch ( argv[i][1] ) {
			case 's' :
			case 'S' :
				if ( argv[i][2] == '\0' ) {
					startFrame = atof( argv[i+1] ); 
				} else {
					printf("Improperly formatted switch %s\n", argv[i]);
					return 0;
				}
				break;
			case 'e' :
			case 'E' : 
				if ( argv[i][2] == '\0' ) {
					endFrame = atof( argv[i+1] );
				} else {
					printf("Improperly formatted switch %s\n", argv[i]);
					return 0;
				}
				break;
			case 'j' :
			case 'J' : 
				if ( argv[i][2] == '\0' ) {
					jobSize = atoi( argv[i+1] ); 
				} else {
					printf("Improperly formatted switch %s\n", argv[i]);
					return 0;
				}				
				break;
			case 'b' :
			case 'B' : 
				if ( argv[i][2] == '\0' ) {
					byFrame = atof( argv[i+1] ); 
				} else {
					printf("Improperly formatted switch %s\n", argv[i]);
					return 0;
				}					
				break;
			default : 
				printf("Invalid Argument '%s'\n", argv[i]);
				return 0; 
				break;
			}
			i+=2;
		} else {
			if ( ! mayafile ) {
				mayafile = argv[i];
				i++;
			} else {
				printf("Invalid Argument '%s'\n", argv[i]);
				return 0;
			}					
		}
	}
	
#ifdef _DEBUG
	printf("%-20s : %-4.2f\n", "Start", startFrame);
	printf("%-20s : %-4.2f\n", "End", endFrame);
	printf("%-20s : %-4.2f\n", "By", byFrame);
	printf("%-20s : %-4d\n", "Job Size", jobSize);
	printf("%-20s : %s\n", "File", mayafile);
#endif
	
	return 1;
};

// Verifies that command line arguments have been parsed properly and 
// values make sense. Prints warnings if something looks wierd.
void c_render::verifyArguments() {
	
	if (jobSize <= 0 ) {
		printf("Invalid 'Job Size' value: %d\n\n", jobSize);
		usage();
		exit(1);
	} else if ( byFrame < 0 ) {
		printf("Invalid 'By frame' value: %.2f\n\n", byFrame);
		usage();
		exit(1);
	} else if (startFrame < 0) {
		printf("Missing start frame parameter.\n\n");
		usage();
		exit(1);
	} else if (endFrame < 0) {
		printf("Missing end frame parameter.\n\n");
		usage();
		exit(1);
	}
	if (! mayafile)	{
		printf("No Maya file specified.\n");
		exit(1);
	} else if (NULL == fopen( mayafile, "r" ))  {
		
		printf("Can not open file '%s'\n", mayafile);
		exit(1);
	}
};

int c_render::openSceneFile() {
	
	char buffer[7];
	
	if ( mayafile && fopen( mayafile, "r" ) ) {
		MFS = new mfstream( mayafile );
	} else {
		return 0;	// maya file unspecified! (should never happen since 
		// this error would have been detected earlier and
		// program should have exited before this point)
	}
	
	// now determine if it's a maya ascii or maya binary file
	int c = 0, i=0;
	for (i=0; i<6 && c != EOF; i++) {
		MFS->getByte(buffer[i]);
	}
	// put chars back
	for (i=i-1; i>=0; i--) {
		MFS->ungetByte(buffer[i]);
	}
	buffer[6] = '\0';
	if ( strcmp( buffer, "//Maya") == 0) {
		mayaFileType = ASCII;
	} else {
		mayaFileType = BINARY;
	}
#ifdef _DEBUG
	printf("%-20s : %s\n", "Maya File Type", (mayaFileType == ASCII) ? "Ascii" : "Binary" );
#endif
	return 1; //success
}

// public interface to read parseable lines from maya file
int c_render::readNextLine() {
	if ( !MFS ) {
		openSceneFile();
	}
	return getNextStatement();
}

/*
used for parsing ascii files. Grabs blocks of text separated by semicolons, or if they're
CPP style comments, a newline character.
returns 0 if EOF has been reached
*/
int c_render::getNextStatement() {
	bool is_C_Style_Comment = false;
	bool is_CPP_Style_Comment = false;
	bool stop = false;
	
	//=========
	// grab first two chars to see what we're dealing with
	//=========
	char c;
	int status;
	
	status = MFS->getByte(c);
	/* keep whitespace for now
	// eat up leading whitespace
	while ( c == ' ' || c == '\t' ) {
	status = MFS->getByte(c);
	}
	*/
	if ( c != '\n' && status ) {
		buf[0] = c;
	} else {
		buf[0] = c;
		buf[1] = '\0';
		return (status); // duck out if it's just a blank line
	}
	status = MFS->getByte(c);
	if ( status ) {
		buf[1] = c;
	} else {
		buf[1] = c;
		buf[2] = '\0';
		return 0;
	}
	// figure out if we're dealing with a comment
	if ( buf[0] == '/' ) {
		if ( buf[1] == '/' ) {
			is_CPP_Style_Comment = true;
		} else if ( buf[1] == '*' ) {
			is_C_Style_Comment = true;
		}
	}
	int i = 2;
	status = MFS->getByte(c);
	while ( status  && !stop && i < MAX_BUFFER_SIZE-2) { // leave 2 chars extra, one for '\n' and one for null terminator
		if ( is_C_Style_Comment && buf[i-1] == '*' && c == '/' ) {
			stop = true;
		}
		else if ( is_CPP_Style_Comment && c == '\n' ) {
			stop = true;
		} else if ( ( c == ';' ) && ( !is_CPP_Style_Comment && !is_C_Style_Comment ) ) {
			stop = true;
		}
		buf[i] = c;	
		i++;
		status = MFS->getByte(c);
	}
	// this is kind of a cheap hack, but the buffer should be big enough that overflow is extremely unlikely
	if ( i >= MAX_BUFFER_SIZE-2 ) {
		printf("\nMAX_BUFFER_SIZE exceeded, parsing errors may have occured.\nPlease send mail to stolley@cs.wisc.edu.\n");
	}
	
	/*  Keep whitespace for now
	// eat up whitespace
	while ( c == ' ' || c == '\t' ) {
	status = MFS->getByte(c);
	}
	*/
	
	if ( c == '\n' ) {
		buf[i] = c;
		buf[i+1] = '\0';
	} else {
		MFS->ungetByte( c );
		buf[i] = '\0';
	}
	return (1);	
}

/*
Used for parsing maya binaries. Works like unix's strings command

  returns 0 if EOF has been reached
*/
int c_render::getNextString() {
	int status;
	char c;
	int strlength = 0;
	
	status = MFS->getByte( c );
	
	while (status && strlength < MAX_BUFFER_SIZE-3 ) {
		if (isalpha(c) ) {
			buf[strlength] = c;
			strlength++;
		} else if ( (c >= 32 && c <= 126) && strlength > 0 ) { // first character must be alpha
			buf[strlength] = c;
			strlength++;
		} else if (strlength < 3 ) {
			
			// write what we have so far out to a file and start over looking for a string
			int i;
			for ( i=0; i<strlength; i++ ) {
				MFS->writeByte( buf[i] );
			}
			MFS->writeByte(c);
			strlength = 0;			
		} else { // otherwise we have a string so put back the char we have, stop and return it
			MFS->ungetByte(c);
			break;
		}
		status = MFS->getByte( c );
	}
	
	buf[strlength] = '\0';
	
	return ( status );
}

int c_render::grabFileRefs_MA() {
	
	char* loc = strstr( buf, MA_FILEREF_TAG );
	
	if ( loc == NULL ) {
		return 0;
	} else {
		loc += strlen( MA_TEXTURE_TAG ); // skip tag text
		char* cur = strrchrq( loc, ' ' ); // get last token in file statment
		
		//first parse out file path and add to list of file refs		 
		char* file_ref_str = new char[strlen(cur)+1];
		strcpy(file_ref_str, cur); // copy file path to separate string
		trim( file_ref_str ); // clean it up
		addToFilerefList( file_ref_str );
		
		
		// now just get the filename, and prepend a tilde (~) to it so we have a temporary file name
		char* filename_ptr = getFilename( file_ref_str ); // get a pointer to just the filename
		char* filename = new char[ strlen(filename_ptr) +3 ];
		strcpy( filename, "~" );
		strcat( filename, filename_ptr );
		
		writeToTranslationLog( file_ref_str, filename ); // keep a record of what files have been translated to what names
		
		addToTransferFileList( filename ); // the temp file gets tranferred with the job
		
		// before writing, snip off the file path part of the buffer because we're re-writing it
		*cur = '\0';
		
		// now write statment to the new scene file
		MFS->writeString( buf );
		MFS->writeString( " \"" ); 
		MFS->writeString( filename ); 
		MFS->writeString("\";\n" ); 	// this might not be needed
		
		return 1;
	}
}

// returns 1 if texture is found, 0 if not, -1 if error
int c_render::grabTextureRefs_MA() {
	
	char* loc = strstr( buf, MA_TEXTURE_TAG );
	
	if ( loc == NULL ) {
		return 0;
	} else {
		
		loc += strlen( MA_TEXTURE_TAG );
		
		// parse out file path, add to list of textures
		char *cur = strrchrq( buf, ' ' );
		trim( cur );
		char* tex_ref_str = new char[strlen(cur)+1];
		strcpy(tex_ref_str, cur);
		
		// add file reference to list so that we can copy it
		addToTextureList( tex_ref_str );
		
		// now rewrite the reference so that it points to the current directory
		char* dest = getFilename( tex_ref_str ); // grab filename
		char* filename = new char[ strlen( dest ) +1 ];
		strcpy( filename, dest );
		addToTransferFileList( filename ); // transfer texture with job
		writeToTranslationLog( tex_ref_str, filename ); // note translation in log
		
		strcpy( buf, MA_TEXTURE_TAG);
		strcat( buf, " (\"" );
		strcat( buf, filename );
		strcat( buf, "\");\n" );
		
		MFS->writeString( buf );
		//		printf("%s", buf);
		return 1;
	}
	
}

int c_render::grabFileRefs_MB() {
	
	
	
	if ( strstr(buf, MB_FILEREF_TAG) && ( strlen(buf)-strlen(MB_FILEREF_TAG) < 2) ) { //string in buf is about the same size as FILEREF_TAG
																					  // it's a weak attempt to prevent erroneous parsing
		MFS->writeString( buf ); // write string to output file
		
		if ( getNextString() ) {
			
			// first add original file ref path to list so it can be parsed later
			char* file_ref_str = new char[strlen(buf)+1];
			strcpy(file_ref_str, buf);
			trim( file_ref_str );
			addToFilerefList( file_ref_str );
			
			// next parse out filename, alter path to file reference so it's in the cwd
			char* filename_ptr = getFilename( buf ); // get a pointer to just the filename
			char* filename = new char[ strlen(filename_ptr) +3 ];
			strcpy( filename, "~" );
			strcat( filename, filename_ptr );
			
			writeToTranslationLog( file_ref_str, filename ); // keep a record of what files have been translated to what names
			addToTransferFileList( filename ); // the temp file gets tranferred with the job
			
			changeFixedLengthString( buf, filename );
			
			// now just write out the string to the new file 
			MFS->writeString( buf ); // write string to output file
			
			return 1;
			
		} else { return 0; }
	} else { return 0; }
}

// we need to be careful with binary files not to change the length
// of strings that we edit. So this guy copies one string to another, 
// but preserves the length of the string by padding unused space with
// space characters. The two strings need not be different arrays.
void changeFixedLengthString( char* oldstr, char* newstr ) {
	int i;
	for (i=0; i<strlen( newstr ); i++ ) {
		
		if ( oldstr[i] == '\0' ) { 
			return;
		} else {
			oldstr[i] = newstr[i];
		}
	}
	while ( oldstr[i] != '\0' ) {
		oldstr[i] = ' ';
		i++;
	}
}

int c_render::grabTextureRefs_MB() {
	
	//	printf("%s\n", buf);
	
	if ( strstr(buf, MB_TEXTURE_TAG) && (strlen(buf)-strlen(MB_TEXTURE_TAG) < 2) ) { // if texture tag is found
																					// and the string is approximately the same size as the texture tag
																					// Trying to avoid erroneous parsing
		
		MFS->writeString( buf ); // write string to output file
		
		if ( getNextString() ) {
			
			
			// first save the originam path of the texture in the texture list
			char* tex_ref_str = new char[strlen(buf)+1];
			strcpy(tex_ref_str, buf);
			trim( tex_ref_str );
			addToTextureList( tex_ref_str );
			
			// next parse out the filename 
			char* filename_ptr = getFilename( buf );
			char* filename = new char[strlen( filename_ptr )+1];
			strcpy( filename, filename_ptr );
			
			addToTransferFileList( filename ); // transfer texture with job
			writeToTranslationLog( tex_ref_str, filename ); // note translation in log
			
			changeFixedLengthString( buf, filename ); //trunc the filename and pad with spaces
			
			MFS->writeString( buf ); // write string to output file
			
			return 1;
		}
	} 
	return 0;
}

// trims off unneeded symbols from both sides of the string
void c_render::trim( char *str ) {
	int start_index = 0;
	int end_index = strlen( str ) -1;
	
	char* ptr = NULL;
	
	// strip end of string
	for (;;) {
		ptr = str + end_index;
		if ( isJunkCharacter( *ptr ) ) {
			end_index--; continue; 
		} else {
			break;
		}
	}
	
	
	*(ptr+1) = '\0';
	
	// strip front of string
	ptr = str;
	int i;
	for ( i=0; i<strlen(str) && isJunkCharacter(str[i]); i++) {
		ptr++;
	}
	
	// now shift the array left if any leading chars have been stripped
	if ( ptr != str ) {
		for (i=0;i<strlen(ptr);i++) {
			str[i] = ptr[i];
		}
		str[i] = '\0';
	}	
}

char* c_render::strrchrq( const char* str, int delim ) {
	bool inQuotedString = false;
	
	char* ptr = (char *) str;
	char* last_ptr = NULL;
	
	for (; *ptr != '\0'; ptr++) {
		if ( (*ptr == delim) && !inQuotedString ) {
			last_ptr = ptr;
		} else if ( *ptr == '"' )	{
			inQuotedString = ! inQuotedString;
		} 		
	}
	return last_ptr;
}

// **don't ever call this recursively...it will make a mess**
// This is an abstraction method meant to handle all of the function calls
// required to parse the maya project and setup a condor submit file.
int c_render::processFile() {
	
	parseDeps();
	// add current file to list of files to include with the job
	file_refs.push_back( mayafile ); // this is why you can't call recursively
	char* transfer = new char[ strlen( MFS->getOutputFileName() ) +1 ];
	strcpy( transfer, MFS->getOutputFileName() );
	addToTransferFileList( transfer );
	writeToTranslationLog( MFS->getInputFileName(), MFS->getOutputFileName() ); // note translation in log
	
	
	// copy texture files to current directory
	int i=0;
	for (i=0; i<texture_ref_cnt; i++ ) {
		char* dest = getFilename( texture_refs[i] );
		/*		char* transfer = new char[ strlen( dest ) +1 ];
		strcpy( transfer, dest );
		addToTransferFileList( transfer );
		*/ 
		if ( fopen( dest, "r" ) ) {
				printf("File '%s' found in current directory.\n", dest);
		} else {
			if (fileCopy(texture_refs[i], dest) ) {
				printf("File '%s' copied to current directory.\n", dest);
			}
		}
	}
	// build submit file and return the results 	
	return buildSubmitFile();
}

int c_render::parseDeps() {
	int result = 0, i = 0, j = 0;
	if (! openSceneFile() ) {
		printf("Unable to open '%s'\n", mayafile);
		exit(0);
	} else {
		printf("Examining '%s'.\n", mayafile);
	}
	if (mayaFileType == ASCII) {
		while ( readNextLine() ) { // EOF hasn't been reached
			int r1 = grabFileRefs_MA();
			int r2 = grabTextureRefs_MA();
			if (r1 || r2) {
				// the string was processed and wrote to the stream already
			} else {
				// didn't find anything interesting in the string,
				// so write it to the output file now
				MFS->writeString( buf );
			}
		}
	} else {  // binary files
		while (getNextString() ) {
			int r1 = grabFileRefs_MB();	// returns 1 if it found something
			int r2 = grabTextureRefs_MB(); // returns 1 if it found something
			if (r1 || r2) {
				// the string was processed and wrote to the stream already
			} else {
				// didn't find anything interesting in the string,
				// so write it to the output file now
				MFS->writeString( buf );
			}
		}
	}
	MFS->flushOutputBuffer();
	//		delete MFS; // destructor will flush remaining buffered data
	
	for (i=0; i<file_ref_cnt; i++ ) {
		fixFilePathForMaya(file_refs[i]);
	}
	for (i=0; i<texture_ref_cnt; i++ ) {
		fixFilePathForMaya(texture_refs[i]);
	}
	
	// now comes the tricky part of recursing through the files refs for more
	// textures and file refs
	
	if ( file_ref_cnt > 0 ) {
		for ( i=0; i<file_ref_cnt; i++ ) {
			c_render *ref = new c_render(file_refs[i]);
			ref->parseDeps();
			
			for(j=0; j<ref->texture_ref_cnt; j++) {
				this->addToTextureList( ref->texture_refs[j] );
				char* temp = getFilename( ref->texture_refs[j] );
				char* str = new char[strlen(temp)+1];
				strcpy( str, temp );
				addToTransferFileList( str );					  
			}
			
			ref->texture_ref_cnt = 0;	// we don't want ref's destructor to wipe out
			// our texture dep list!
			
			for(j=0; j<ref->file_ref_cnt; j++) {
				addToFilerefList( ref->file_refs[j] );
			}
			
			ref->file_ref_cnt = 0;	// we don't want ref's destructor to wipe out
			// our file dep list!
			delete ref;				
		}
	}
	return 1;
}
int c_render::printDeps() {
	int i;
	printf("\nTexture Dependencies: %s\n", (texture_ref_cnt) ? "" : "(None)");
	for(i=0; i<texture_ref_cnt; i++) {
		printf("\t%s\n", texture_refs[i]);
	}
	printf("\nFile Dependencies: %s\n", (file_ref_cnt) ? "" : "(None)");
	for(i=0; i<file_ref_cnt; i++) {
		printf("\t%s\n", file_refs[i]);
	}
	
	return 1;
}

int c_render::fixFilePathForMaya( char* & path ) {
	fixPathSeparators( path ); // fix them initially since they're fresh out of the maya file
	if ( isRelativePath( path ) ) {
		// save original working directory
		char* org_cwd = new char[MAXPATHLEN];
		getcwd(org_cwd, MAXPATHLEN);
		
		// change to parent directory containing maya file 
		char* tempmayafile = new char[strlen(mayafile) + 1];
		char* temp = NULL;
		strcpy( tempmayafile, mayafile );
		
		if ( ! getParentDirectory( tempmayafile ) ) { // hopefully will pop off maya file name
			temp = new char[MAXPATHLEN];
			getcwd(temp, MAXPATHLEN);
			strcat(temp, "\\..\\");
			//strcat(temp, tempmayafile); // relative paths in maya always refer to _parent_ directory not current directory			
		} else {
			temp = new char[strlen(tempmayafile)+MAXPATHLEN+5];
			strcpy(temp, tempmayafile ); // relative paths in maya always refer to _parent_ directory not current directory			
			char parent[] = { PATH_SEPARATOR, '.', '.', PATH_SEPARATOR, '\0' };
			strcat(temp, parent );
		}
		chdir( temp );
		delete[] tempmayafile;
		delete[] temp;
		
		// fix path
		char* cwd = new char[MAXPATHLEN];
		getcwd(cwd, MAXPATHLEN);
		canonicalizePath( cwd, path );
		delete[] cwd;
		chdir( org_cwd ); // restore working directory
		delete[] org_cwd;
		return 1;
	} else {
		return 0;
	}
}

int c_render::isRelativePath( const char* path ) {
	char* cur = (char*) path;
	while ( isWhiteSpace( *cur ) ) { cur++; } // trim off excess whitespace
	if ( isalpha( cur[0] ) && 
		( cur[1] == ':' ) &&
		( isPathSeparator( cur[2]) )) { // path starts with [drive letter]:\\ not next line
		return 0;
	} else if (isPathSeparator(*cur)) { // path starts with root ( e.g. /foo/dir )
		return 0;
	} else { // assume path is a relative path
		return 1;
	}
	
}

// makes relative paths absolute based on the specified working directory
// if you call this guy you still have to free the path pointer
int c_render::canonicalizePath( const char* cwd, char* &path ) {
	// if path string starts with C:\ or [path_separator] assume it is an absolute path
	char* cur = path;
	while ( isWhiteSpace( *cur ) ) { cur++; } // trim off excess whitespace
	if ( isRelativePath( path ) ) {
		char* abspath = new char[MAXPATHLEN+1];
		strcpy( abspath, cwd );		
		if (! isPathSeparator(abspath[strlen(abspath)-1]) ) {
			strcat( abspath, "\\");
		}
		strcat( abspath, path);
		delete[] path;
		path = NULL;
		path = abspath;
		fixPathSeparators( path );
		return 1;
	}
	return 0;
}

void c_render::fixPathSeparators( char* &path ) {
	
	for (int i = 0; i < strlen(path); i++) {
		if (isPathSeparator(path[i]) ) {
			path[i] = PATH_SEPARATOR;
		}
	}
}

// pops off one level of a path, or returns 0 if there's nothing left to pop
int c_render::getParentDirectory( char* path ) {
	char *parent = strrchr(path, PATH_SEPARATOR);
	if ( parent ) {
		*parent = '\0';
		return 1;
	} else {
		return 0;
	}
}

int c_render::buildSubmitFile() {
	int i;
	char c;
	// open header file and open a new submit file
	//	mfstream* submit_file = new mfstream( CONDOR_SUBMIT_HDR_FILE, CONDOR_SUBMIT_FILE ); // open header, make new submit file
	mfstream* submit_file = new mfstream( Condor_submit_hdr_file, CONDOR_SUBMIT_FILE ); // open header, make new submit file
	
	if (! submit_file->ready() ) {
		printf("ERROR: Unable to create Condor Submit File.\n");
		return 0;
	} else {	
		
		// copy header to new submit file
		while ( submit_file->getByte(c) ) {
			submit_file->writeByte( c );
		}
		
		// fill in transfer_input_files parameter in submit file
		if ( transfer_files_cnt ) {
			submit_file->writeString( "transfer_input_files = " );
			submit_file->writeString( transfer_files[0] );
			for (i=1; i<transfer_files_cnt; i++) {
				submit_file->writeString( ", " );
				submit_file->writeString( transfer_files[i] );			
			}
		}
		
		// now fill in the arguments for each job and corresponding queue statments
		float jobIncr = (jobSize * byFrame) - byFrame;
		float curFirstFrame = startFrame;
		float curLastFrame = startFrame-byFrame;
		char numstr[100];
		char* numstrptr = numstr;
		
		
		do {	
			
			submit_file->writeString("\n\nArguments = -rd . -im ");
			submit_file->writeString(output_file_name);
			
			submit_file->writeString(" -s ");
			curFirstFrame = curLastFrame+byFrame;
			sprintf(numstrptr, "%.4f", curFirstFrame);
			submit_file->writeString( numstrptr );
			
			submit_file->writeString(" -e ");
			curLastFrame = curFirstFrame + jobIncr;
			if ( curLastFrame > endFrame ) {
				curLastFrame = endFrame; 
			}
			sprintf(numstrptr, "%.4f", curLastFrame);
			submit_file->writeString( numstrptr );
			
			submit_file->writeString(" -b ");
			sprintf(numstrptr, "%.4f", byFrame);
			submit_file->writeString( numstrptr );
			
			submit_file->writeString(" ");
			submit_file->writeString(MFS->getOutputFileName() ); //give the renderer the name of the scene file
			
			submit_file->writeString("\nQueue\n");
		} while ( curLastFrame < endFrame );
		
		delete submit_file;
		return 1;
	}
}

int c_render::submitToCondor() {
	printf("\n\nCalling condor_submit:\n");
	return system("condor_submit condor_render.sub");
}

char* getFilename(char* path ) {
	char* filename = strrchr( path, '\\' ); // try both kinds of separators
	if ( ! filename ) { filename = strrchr( path, '/' ); } 
	if ( filename ) {
		return filename+1;
	} else {
		return (char*) path;
	}
}



int main( int argc, char *argv[] )
{
	int result = 0, ch = 0;
	c_render *foo;

#ifndef PRE_RELEASE
	// this is for the real release of condor_render
	foo = new c_render();
	if ( foo->parseCommandArgs( argc, argv ) )	{
		foo->verifyArguments(); 
	} else { 
		exit(1);
	}
	
#else
	// this is for the 'just show me the dependencies list' version of condor_render
	foo = new c_render(argv[1]);
	if (! foo->mayafile)	{
		printf("No Maya file specified.\n");
		exit(1);
	} else if (NULL == fopen( foo->mayafile, "r" ))  {
		
		printf("Can not open file '%s'\n", foo->mayafile);
		exit(1);
	}
	
#endif
	result = foo->processFile();
	if ( result ) {
		foo->printDeps();
		if (foo->submitToCondor() == 0) {
			printf("\nSubmitted jobs successfully. Have a nice day!\n");
		} else {
			printf("\nCondor_submit failed. Make sure condor_submit is in your search path.\n");
		}
	}
	
	delete foo;
	return 0;
	
};
